<?php

namespace Drupal\charts_highcharts_grouped_categories\Plugin\views\style;

use Drupal\charts\Plugin\views\style\ChartsPluginStyleChart;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;

/**
 * Style plugin to render view as a chart.
 *
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *   id = "chart_highcharts_grouped_categories",
 *   title = @Translation("Chart highcharts grouped categories"),
 *   help = @Translation("Render a chart with grouped categories."),
 *   theme = "views_view_chart_highcharts_grouped_categories",
 *   display_types = { "normal" }
 * )
 */
class GroupedCategoriesPluginStyleChart extends ChartsPluginStyleChart {

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['fields_level_0'] = ['default' => ''];
    $options['fields_level_1'] = ['default' => ''];
    $options['fields_level_2'] = ['default' => ''];

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $data_options = ['' => '- Select -'] + $this->displayHandler->getFieldLabels();

    $form['fields_level_0'] = [
      '#type' => 'select',
      '#title' => $this->t('Level 0'),
      '#options' => $data_options,
      '#weight' => -8,
      '#default_value' => $this->options['fields_level_0'],
      '#description' => $this->t('Select the level 0 field. Example: Cities'),
    ];
    $form['fields_level_1'] = [
      '#type' => 'select',
      '#title' => $this->t('Level 1'),
      '#options' => $data_options,
      '#weight' => -7,
      '#default_value' => $this->options['fields_level_1'],
      '#description' => $this->t('Select the level 1 field. Example: Countries'),
    ];
    $form['fields_level_2'] = [
      '#type' => 'select',
      '#title' => $this->t('Level 2'),
      '#options' => $data_options,
      '#weight' => -6,
      '#default_value' => $this->options['fields_level_2'],
      '#description' => $this->t('Select the level 2 field. Example: Continents'),
    ];
    $form_state->set('default_options', $this->options);
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $chart = parent::render();

    $form_options = $this->options;
    $chart_settings = $this->options['chart_settings'];
    // If the chart is column only.
    if ($chart_settings['library'] == 'highcharts' && $chart_settings['type'] == 'column') {
      // Need to configure at least level 0 and 1.
      if ($form_options['fields_level_0'] != NULL && $form_options['fields_level_1'] != NULL) {
        $level_0_field = $form_options['fields_level_0'];
        $level_1_field = $form_options['fields_level_1'];
        $level_2_field = $form_options['fields_level_2'];
        $chart['#id'] = Html::getId($this->view->id() . '_' . $this->view->current_display);

        $this->renderFields($this->view->result);
        $renders = $this->rendered_fields;

        // Build grouped categories array for level 1.
        $categories = $this->buildCategories($renders, $level_0_field, $level_1_field, []);

        // Build grouped categories array for level 2.
        if ($level_2_field != NULL) {
          $level_2_categories = $this->buildCategories($renders, $level_1_field, $level_2_field, $categories);
        }

        $options['grouped_categories'] = $level_2_field == NULL ? $categories : $level_2_categories;
        $chart['#raw_options'] = $options;
      }
      else {
        \Drupal::logger('Charts_highcharts_grouped_categories')
          ->notice('Select at least Level 0 and Level 1 in the option form.');
      }
    }
    else {
      \Drupal::logger('Charts_highcharts_grouped_categories')
        ->notice('The library should be highcharts and the chart a column type.');
    }

    return $chart;
  }

  /**
   * Processes number value based on field.
   *
   * @param int $number
   *   The number.
   * @param string $field
   *   The field.
   *
   * @return string|null
   *   The value.
   */
  public function processNumberValueFromFieldCustom($number, $field) {
    $value = trim(strip_tags((string)$this->getField($number, $field)));
    if ($value === '' || is_null($value)) {
      $value = NULL;
    }

    return $value;
  }

  /**
   * Get a category by name
   *
   * @param array $categories
   * The categories
   * @param string $name
   * The name
   *@return array|null
   * The category or null if not found
   */
  private function getCategoryByName(array $categories, string $name) {
    $cat = array_filter($categories, fn($cat) => $cat['name'] === $name);
    return reset($cat) ?: [];
  }

  /**
   * @param array $renders
   * The rendered result array
   * @param string $first_level
   * Lower level field
   * @param string $second_level
   * Hight level field
   * @param array $categories
   * The categories of level 1 in case we build for level 2.
   *
   * @return array
   */
  private function buildCategories(array $renders, string $first_level, string $second_level, array $categories): array {
    $cats = [];
    // Build grouped categories array.
    foreach ($renders as $row_number => $row) {
      $value_0 = $this->processNumberValueFromFieldCustom($row_number, $first_level);
      $value_1 = $this->processNumberValueFromFieldCustom($row_number, $second_level);
      $exist = FALSE;
      if (!empty($cats)) {
        foreach ($cats as $index => $category) {
          if ($category['name'] == $value_1) {
            // Handle differently level 1 and level 2 category building.
            // Do not add twice the same category since it is probably multiple in the renders array.
            if ($categories == NULL) {
              $cats[$index]['categories'][] = $value_0;
            }
            else {
              $this->getCategoryByName($cats[$index]['categories'], $value_0) == NULL
                ? $cats[$index]['categories'][] = $this->getCategoryByName($categories, $value_0)
                : NULL;
            }
            $exist = TRUE;
          }
        }
      }
      if (empty($cats) || !$exist) {
        $cats[] = [
          "name" => $value_1,
          "categories" => [$categories == NULL ? $value_0 : $this->getCategoryByName($categories, $value_0)],
        ];
      }
    }

    return $cats;
  }

}
